'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.NumericInput = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _createRef = require('create-react-ref/lib/createRef');

var _createRef2 = _interopRequireDefault(_createRef);

var _withTheme = require('./HOC/withTheme');

var _themes = require('../utils/themes');

var _ = require('.');

var _strings = require('../utils/strings');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

// external libraries

// $FlowFixMe


// internal utility functions


// import constants


// TODO: make this configurable (generalize handling commas and dots in other languages!)
var LOCALE = 'en-US';

var NumericInputBase = function (_Component) {
  _inherits(NumericInputBase, _Component);

  function NumericInputBase(props) {
    _classCallCheck(this, NumericInputBase);

    var _this = _possibleConstructorReturn(this, (NumericInputBase.__proto__ || Object.getPrototypeOf(NumericInputBase)).call(this, props));

    _this._hasInputBeenChanged = false;

    _this.onChange = function (event) {
      event.preventDefault();
      var _this$props = _this.props,
          value = _this$props.value,
          onChange = _this$props.onChange,
          disabled = _this$props.disabled;

      if (disabled) {
        return;
      }
      var result = _this.processValueChange(event.nativeEvent);
      if (result) {
        _this._hasInputBeenChanged = true;
        var hasValueChanged = value !== result.value;
        if (hasValueChanged && onChange) {
          onChange(result.value, event);
        }
        _this.setState({
          inputCaretPosition: result.caretPosition,
          minimumFractionDigits: result.minimumFractionDigits,
          fallbackInputValue: result.fallbackInputValue
        });
      }
    };

    _this.setInputCaretPosition = function (position) {
      var inputElement = _this.inputElement;

      if (!inputElement.current) return;
      var input = inputElement.current;
      input.selectionStart = position;
      input.selectionEnd = position;
    };

    _this.focus = function () {
      var inputElement = _this.inputElement;

      if (!inputElement.current) return;
      inputElement.current.focus();
    };

    _this.onBlur = function () {
      _this.setState({
        fallbackInputValue: null
      });
    };

    var context = props.context,
        numberLocaleOptions = props.numberLocaleOptions,
        themeId = props.themeId,
        theme = props.theme,
        themeOverrides = props.themeOverrides;

    _this.inputElement = (0, _createRef2.default)();
    var minimumFractionDigits = numberLocaleOptions ? numberLocaleOptions.minimumFractionDigits : 0;
    _this.state = {
      composedTheme: (0, _themes.composeTheme)((0, _themes.addThemeId)(theme || context.theme, themeId), (0, _themes.addThemeId)(themeOverrides, themeId), context.ROOT_THEME_API),
      minimumFractionDigits: minimumFractionDigits || 0,
      inputCaretPosition: 0,
      fallbackInputValue: null
    };
    return _this;
  }

  _createClass(NumericInputBase, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var inputElement = this.inputElement;
      var autoFocus = this.props.autoFocus;

      if (autoFocus) {
        this.focus();
        if (inputElement && inputElement.current) {
          this.setState({
            inputCaretPosition: inputElement.current.selectionStart
          });
        }
      }
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      (0, _themes.didThemePropsChange)(this.props, nextProps, this.setState.bind(this));
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate(prevProps, prevState) {
      var value = this.props.value;
      var inputCaretPosition = this.state.inputCaretPosition;

      var hasValueBeenChanged = value !== prevProps.value;
      var hasCaretBeenChanged = inputCaretPosition !== prevState.inputCaretPosition;
      if (this._hasInputBeenChanged || hasValueBeenChanged || hasCaretBeenChanged) {
        this.setInputCaretPosition(inputCaretPosition);
      }
      this._hasInputBeenChanged = false;
    }
  }, {
    key: 'processValueChange',


    /**
     * 1. Handle edge cases that don't need further processing
     * 2. Clean the given value
     * 3. Final processing
     */
    value: function processValueChange(event) {
      var changedCaretPosition = event.target.selectionStart;
      var valueToProcess = event.target.value;
      var inputType = event.inputType;
      var _props = this.props,
          value = _props.value,
          allowSigns = _props.allowSigns;

      var fallbackInputValue = this.state.fallbackInputValue || '';
      var isBackwardDelete = inputType === 'deleteContentBackward';
      var isForwardDelete = inputType === 'deleteContentForward';
      var isDeletion = isForwardDelete || isBackwardDelete;
      var isInsert = inputType === 'insertText';
      var deleteCaretCorrection = isBackwardDelete ? 0 : 1;
      var validInputRegex = allowSigns ? VALID_INPUT_SIGNS_REGEX : VALID_INPUT_NO_SIGNS_REGEX;
      var valueHasLeadingZero = /^0[1-9]/.test(valueToProcess);

      /**
       * ========= HANDLE HARD EDGE-CASES =============
       */
      // Case: invalid characters entered -> refuse!
      if (!validInputRegex.test(valueToProcess)) {
        return {
          caretPosition: changedCaretPosition - 1,
          fallbackInputValue: fallbackInputValue,
          minimumFractionDigits: this.state.minimumFractionDigits,
          value: value
        };
      }

      // Case: Everything was deleted -> reset state
      if (valueToProcess === '') {
        return {
          value: null,
          caretPosition: 0,
          fallbackInputValue: null,
          minimumFractionDigits: 0
        };
      }

      // Case: value is the same as the fallback (which is always shown if defined)
      if (valueToProcess === this.state.fallbackInputValue) return null;

      // Case: Just minus sign was entered
      if (valueToProcess === '-') {
        return {
          value: null,
          caretPosition: 1,
          fallbackInputValue: '-',
          minimumFractionDigits: 0
        };
      }

      /**
       * ========= CLEAN THE INPUT =============
       */

      // Options
      var propsMinimumFractionDigits = this.getMinimumFractionDigits();
      var maximumFractionDigits = this.getMaximumFractionDigits();
      var numberLocaleOptions = this.getDynamicLocaleOptions();

      // Current value
      var currentNumber = value;
      var currentValue = value != null ? value.toLocaleString(LOCALE, numberLocaleOptions) : fallbackInputValue;
      var currentNumberOfDots = getNumberOfDots(currentValue);
      var hadDotBefore = currentNumberOfDots > 0;

      // New Value
      var newValue = valueToProcess;
      var newCaretPosition = changedCaretPosition;
      var newNumberOfDots = getNumberOfDots(newValue);

      // Case: A second decimal point was added somewhere
      if (hadDotBefore && newNumberOfDots === 2) {
        var oldFirstDotIndex = currentValue.indexOf('.');
        var newFirstDotIndex = newValue.indexOf('.');
        var wasDotAddedBeforeOldOne = newFirstDotIndex < oldFirstDotIndex;
        // Remove the second decimal point and set caret position
        newValue = (0, _strings.removeCharAtPosition)(newValue, wasDotAddedBeforeOldOne ? newValue.lastIndexOf('.') : oldFirstDotIndex);
        newCaretPosition = newValue.indexOf('.') + 1;
      }

      newValue = truncateToPrecision(newValue, maximumFractionDigits);

      /**
       * ========= PROCESS CLEANED INPUT =============
       */
      var numberOfFractionDigits = getFractionDigits(newValue).length;
      var dynamicMinimumFractionDigits = Math.min(Math.max(propsMinimumFractionDigits, numberOfFractionDigits), maximumFractionDigits);
      var newNumber = getValueAsNumber(newValue, maximumFractionDigits);

      // Case: Just a dot was entered
      if (valueToProcess === '.') {
        var hasMinFractions = dynamicMinimumFractionDigits > 0;
        return {
          value: 0,
          caretPosition: 2,
          fallbackInputValue: hasMinFractions ? null : '0.',
          minimumFractionDigits: dynamicMinimumFractionDigits
        };
      }

      // Case: Dot was added at the beginning of number
      if (newValue.charAt(0) === '.') {
        var newCaretPos = isInsert ? 2 : 1;
        return {
          value: newNumber,
          caretPosition: newCaretPos,
          minimumFractionDigits: dynamicMinimumFractionDigits
        };
      }

      // Case: Invalid change has been made -> ignore it
      if (newNumber == null) {
        var deleteAdjustment = isBackwardDelete ? 0 : 1; // special cases when deleting dot
        var insertAdjustment = -1; // don't move caret if numbers are "inserted"
        return {
          caretPosition: changedCaretPosition + (isDeletion ? deleteAdjustment : insertAdjustment),
          fallbackInputValue: fallbackInputValue,
          minimumFractionDigits: dynamicMinimumFractionDigits,
          value: currentNumber
        };
      }

      var localizedNewNumber = newNumber.toLocaleString(LOCALE, {
        minimumFractionDigits: dynamicMinimumFractionDigits
      });

      // Case: Dot was added at the end of number
      if (!isDeletion && newValue.charAt(newValue.length - 1) === '.') {
        return {
          value: newNumber,
          caretPosition: changedCaretPosition,
          fallbackInputValue: propsMinimumFractionDigits > 0 ? null : localizedNewNumber + '.',
          minimumFractionDigits: 0
        };
      }

      // Case: Dot was deleted with minimum fraction digits constrain defined
      var hasFractions = this.getMinimumFractionDigitsProp() != null;
      var wasDotRemoved = hadDotBefore && !newNumberOfDots;
      if (wasDotRemoved && hasFractions && !isInsert) {
        return {
          caretPosition: newCaretPosition + deleteCaretCorrection,
          fallbackInputValue: null,
          minimumFractionDigits: dynamicMinimumFractionDigits,
          value: currentNumber
        };
      }

      // Case: Valid change has been made
      var hasNumberChanged = value !== newNumber;
      var commasDiff = getNumberOfCommas(localizedNewNumber) - getNumberOfCommas(newValue);
      var haveCommasChanged = commasDiff > 0;
      var onlyCommasChanged = !hasNumberChanged && haveCommasChanged;
      var leadingZeroCorrection = valueHasLeadingZero ? -1 : 0;
      var caretCorrection = (onlyCommasChanged ? deleteCaretCorrection : commasDiff) + leadingZeroCorrection;
      return {
        caretPosition: Math.max(newCaretPosition + caretCorrection, 0),
        fallbackInputValue: null,
        minimumFractionDigits: dynamicMinimumFractionDigits,
        value: newNumber
      };
    }
  }, {
    key: 'getMinimumFractionDigitsProp',
    value: function getMinimumFractionDigitsProp() {
      var numberLocaleOptions = this.props.numberLocaleOptions;

      if (!numberLocaleOptions) return null;
      var minimumFractionDigits = numberLocaleOptions.minimumFractionDigits;

      return minimumFractionDigits || null;
    }
  }, {
    key: 'getMinimumFractionDigits',
    value: function getMinimumFractionDigits() {
      return this.getMinimumFractionDigitsProp() || 0;
    }
  }, {
    key: 'getDynamicMinimumFractionDigits',
    value: function getDynamicMinimumFractionDigits() {
      var minimumFractionDigitsProp = this.getMinimumFractionDigits();
      return Math.max(this.state.minimumFractionDigits, minimumFractionDigitsProp || 0);
    }
  }, {
    key: 'getMaximumFractionDigits',
    value: function getMaximumFractionDigits() {
      var numberLocaleOptions = this.props.numberLocaleOptions;

      var minimumFractionDigits = this.getDynamicMinimumFractionDigits();
      var maximumFractionDigits = numberLocaleOptions && numberLocaleOptions.maximumFractionDigits != null ? numberLocaleOptions.maximumFractionDigits : 3;
      return Math.max(minimumFractionDigits, maximumFractionDigits);
    }
  }, {
    key: 'getDynamicLocaleOptions',
    value: function getDynamicLocaleOptions() {
      return Object.assign({}, this.props.numberLocaleOptions, {
        minimumFractionDigits: this.getDynamicMinimumFractionDigits()
      });
    }
  }, {
    key: 'getLocalizedNumber',
    value: function getLocalizedNumber(value) {
      return convertNumberToLocalizedString(value, LOCALE, this.getDynamicLocaleOptions());
    }
  }, {
    key: 'render',
    value: function render() {
      // destructuring props ensures only the "...rest" get passed down
      var _props2 = this.props,
          context = _props2.context,
          error = _props2.error,
          numberLocaleOptions = _props2.numberLocaleOptions,
          onChange = _props2.onChange,
          skin = _props2.skin,
          theme = _props2.theme,
          themeOverrides = _props2.themeOverrides,
          value = _props2.value,
          rest = _objectWithoutProperties(_props2, ['context', 'error', 'numberLocaleOptions', 'onChange', 'skin', 'theme', 'themeOverrides', 'value']);

      var InputSkin = skin || context.skins[_.IDENTIFIERS.INPUT];

      var localizedInput = value != null ? this.getLocalizedNumber(value) : '';
      var inputValue = this.state.fallbackInputValue ? this.state.fallbackInputValue : localizedInput;

      return _react2.default.createElement(InputSkin, _extends({
        error: error,
        inputRef: this.inputElement,
        onChange: this.onChange,
        theme: this.state.composedTheme,
        value: inputValue,
        onBlur: this.onBlur
      }, rest));
    }
  }]);

  return NumericInputBase;
}(_react.Component);

NumericInputBase.displayName = 'NumericInput';
NumericInputBase.defaultProps = {
  allowSigns: true,
  context: (0, _withTheme.createEmptyContext)(),
  readOnly: false,
  theme: null,
  themeId: _.IDENTIFIERS.INPUT,
  themeOverrides: {},
  value: null
};
var NumericInput = exports.NumericInput = (0, _withTheme.withTheme)(NumericInputBase);

// ========= HELPERS ==========

var VALID_INPUT_SIGNS_REGEX = /^([-])?([0-9,.]+)?$/;
var VALID_INPUT_NO_SIGNS_REGEX = /^([0-9,.]+)?$/;
var NUMERIC_INPUT_REGEX = /^([-])?([0-9,]+)?(\.([0-9]+)?)?$/;

var isValidNumericInput = function isValidNumericInput(value) {
  return NUMERIC_INPUT_REGEX.test(value);
};

var isParsableNumberString = function isParsableNumberString(value, requiredPrecision) {
  return (
    // The number of digits is limited in Javascript - so the required precision influence
    // the possible number of integer digits (only 15 digits can be safely represented in total)
    parseFloat(value) >= Number.MIN_SAFE_INTEGER / Math.pow(10, requiredPrecision + 1) && parseFloat(value) <= Number.MAX_SAFE_INTEGER / Math.pow(10, requiredPrecision + 1) && !isNaN(parseFloat(value)) && isFinite(value)
  );
};

var removeCommas = function removeCommas(value) {
  return value.replace(/,/g, '');
};

function parseStringToNumber(value, requiredPrecision) {
  var cleanedValue = removeCommas(value);
  if (!isValidNumericInput(cleanedValue)) return null;
  if (!isParsableNumberString(cleanedValue, requiredPrecision)) return null;
  return parseFloat(cleanedValue);
}

function convertNumberToLocalizedString(num, locale, options) {
  return num != null ? num.toLocaleString(locale, options) : '';
}

function getValueAsNumber(value, requiredPrecision) {
  return typeof value === 'string' ? parseStringToNumber(value, requiredPrecision) : value;
}

function getNumberOfCommas(value) {
  return (value.match(/,/g) || []).length;
}

function getNumberOfDots(value) {
  return (value.match(/\./g) || []).length;
}

function getIntegerDigits(value) {
  var decimalPointIndex = value.indexOf('.');
  if (decimalPointIndex === -1) return value;
  return value.substring(0, decimalPointIndex);
}

function getFractionDigits(value) {
  var decimalPointIndex = value.indexOf('.');
  if (decimalPointIndex === -1) return '';
  return value.substring(decimalPointIndex + 1);
}

function truncateToPrecision(value, precision) {
  var decimalPointIndex = value.indexOf('.');
  if (decimalPointIndex === -1) return value;
  var fractionDigits = removeCommas(getFractionDigits(value));
  return getIntegerDigits(value) + '.' + fractionDigits.substring(0, precision);
}
//# sourceMappingURL=NumericInput.js.map